﻿using fichier1.Data;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace fichier1.Model
{
    public class Partie : IObservable<Partie>
    {
        private  List<string[]> correction;
        private List<string[]> decoupage;
        private int motCourant;
        private readonly BddContext context;
        private List<IObserver<Partie>> observers;
        public int nbMots;

        public Partie(int nbMots, float complexite, BddContext context)
        {
            this.nbMots = nbMots;
            this.context = context;
            this.observers = new List<IObserver<Partie>>();
        }

        public void genererMots(int nb, float complexite)
        {
            correction = new List<string[]>();

            Random rnd = new Random();

            Prefixe[] prefixe = context.Prefixes.ToArray();
            Radical[] radical = context.Radicals.ToArray();
            Suffixe[] suffixe = context.Suffixes.ToArray();

            for(int i = 0; i < nb; i++)
            {
                string[] mot = new string[3];
                mot[0] = prefixe[rnd.Next(0, prefixe.Length)].Pre;
                mot[1] = radical[rnd.Next(0, radical.Length)].Rad;
                mot[2] = suffixe[rnd.Next(0, suffixe.Length)].Suf;

                correction.Add(mot);
            }
        }

        public void formaterPropositions()
        {
            decoupage = new List<string[]>();
            for (int i = 0; i < correction.Count; i++)
            {
                string mot = "";
                for (int j = 0; j < correction[i].Length; j++)
                {
                    mot += correction[i][j];
                }
                string[] m = new string[1];
                m[0] = mot;
                decoupage.Add(m);
            }
        }

        public string[] getMot(int nb)
        {
            return decoupage[nb];
        }

        public void setMot(int nb, string[] mot)
        {
            decoupage[nb] = mot;
        }

        public int compterPoints()
        {
            // TODO: 
            return 0;
        }

        public string[] getCorrection(int nb)
        {
            return correction[nb];
        }

        public string[] getMotCourant()
        {
            return getMot(motCourant);
        }

        public void validerMotCourant(string[] mot)
        {
            setMot(motCourant, mot);
            motCourant++;
            if (motCourant < correction.Count)
            {
                foreach(IObserver<Partie> obs in observers)
                {
                    obs.OnNext(this);
                }
            }
            else
            {
                foreach (IObserver<Partie> obs in observers)
                {
                    obs.OnCompleted();
                }
            }
        }

        public IDisposable Subscribe(IObserver<Partie> observer)
        {
            observers.Add(observer);
            return null;
        }
    }
}
